Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/ArchitectureAlteringOperators/SubroutineDeleter.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/ArchitectureAlteringOperators/SubroutineDeleter.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/ArchitectureAlteringOperators/SubroutineDeleter.cs	(revision 3369)
@@ -68,5 +68,5 @@
 
       // remove references to deleted function
-      foreach (var subtree in symbolicExpressionTree.Root.SubTrees) {
+      foreach (var subtree in symbolicExpressionTree.Root.SubTrees.OfType<SymbolicExpressionTreeTopLevelNode>()) {
         var matchingInvokeSymbol = (from symb in subtree.Grammar.Symbols.OfType<InvokeFunction>()
                                     where symb.FunctionName == selectedDefunBranch.FunctionName
@@ -99,8 +99,9 @@
         int minPossibleHeight = invocationCutPoint.Parent.Grammar.GetMinExpressionDepth(selectedSymbol);
         int maxHeight = Math.Max(minPossibleHeight, invocationCutPoint.ReplacedChild.GetHeight());
-
-        replacementTree = ProbabilisticTreeCreator.PTC2(random, invocationCutPoint.Parent.Grammar, selectedSymbol, maxSize, maxHeight, 0, 0);
+        replacementTree = selectedSymbol.CreateTreeNode();
         invocationCutPoint.Parent.RemoveSubTree(invocationCutPoint.ReplacedChildIndex);
         invocationCutPoint.Parent.InsertSubTree(invocationCutPoint.ReplacedChildIndex, replacementTree);
+
+        ProbabilisticTreeCreator.PTC2(random, replacementTree, maxSize, maxHeight, 0, 0);
 
         invocationCutPoint = (from node in symbolicExpressionTree.IterateNodesPrefix()
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/ArchitectureAlteringOperators/SubroutineDuplicater.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/ArchitectureAlteringOperators/SubroutineDuplicater.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/ArchitectureAlteringOperators/SubroutineDuplicater.cs	(revision 3369)
@@ -72,5 +72,5 @@
       duplicatedDefunBranch.Grammar = (ISymbolicExpressionGrammar)selectedBranch.Grammar.Clone();
       // add an invoke symbol for each branch that is allowed to invoke the original function
-      foreach (var subtree in symbolicExpressionTree.Root.SubTrees) {
+      foreach (var subtree in symbolicExpressionTree.Root.SubTrees.OfType<SymbolicExpressionTreeTopLevelNode>()) {
         var matchingInvokeSymbol = (from symb in subtree.Grammar.Symbols.OfType<InvokeFunction>()
                                     where symb.FunctionName == selectedBranch.FunctionName
@@ -79,16 +79,17 @@
           GrammarModifier.AddDynamicSymbol(subtree.Grammar, subtree.Symbol, duplicatedDefunBranch.FunctionName, duplicatedDefunBranch.NumberOfArguments);
         }
-      }
-      // for all invoke nodes of the original function replace the invoke of the original function with an invoke of the new function randomly
-      var originalFunctionInvocations = from node in symbolicExpressionTree.IterateNodesPrefix().OfType<InvokeFunctionTreeNode>()
-                                        where node.Symbol.FunctionName == selectedBranch.FunctionName
-                                        select node;
-      foreach (var originalFunctionInvokeNode in originalFunctionInvocations) {
-        var newInvokeSymbol = (from symb in originalFunctionInvokeNode.Grammar.Symbols.OfType<InvokeFunction>()
-                               where symb.FunctionName == duplicatedDefunBranch.FunctionName
-                               select symb).Single();
-        // flip coin wether to replace with newly defined function
-        if (random.NextDouble() < 0.5) {
-          originalFunctionInvokeNode.Symbol = newInvokeSymbol;
+        // in the current subtree:
+        // for all invoke nodes of the original function replace the invoke of the original function with an invoke of the new function randomly
+        var originalFunctionInvocations = from node in subtree.IterateNodesPrefix().OfType<InvokeFunctionTreeNode>()
+                                          where node.Symbol.FunctionName == selectedBranch.FunctionName
+                                          select node;
+        foreach (var originalFunctionInvokeNode in originalFunctionInvocations) {
+          var newInvokeSymbol = (from symb in subtree.Grammar.Symbols.OfType<InvokeFunction>()
+                                 where symb.FunctionName == duplicatedDefunBranch.FunctionName
+                                 select symb).Single();
+          // flip coin wether to replace with newly defined function
+          if (random.NextDouble() < 0.5) {
+            originalFunctionInvokeNode.Symbol = newInvokeSymbol;
+          }
         }
       }
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Crossovers/SubtreeCrossover.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Crossovers/SubtreeCrossover.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Crossovers/SubtreeCrossover.cs	(revision 3369)
@@ -61,5 +61,5 @@
       SymbolicExpressionTreeNode crossoverPoint0;
       int replacedSubtreeIndex;
-      SelectCrossoverPoint(random, parent0, internalCrossoverPointProbability, out crossoverPoint0, out replacedSubtreeIndex);
+      SelectCrossoverPoint(random, parent0, internalCrossoverPointProbability, maxTreeSize - 1, maxTreeHeight - 1, out crossoverPoint0, out replacedSubtreeIndex);
 
       // calculate the max size and height that the inserted branch can have 
@@ -67,10 +67,9 @@
       int maxInsertedBranchHeight = maxTreeHeight - GetBranchLevel(parent0.Root, crossoverPoint0);
 
-      var allowedBranches = from branch in parent1.Root.IterateNodesPrefix()
-                            where branch.GetSize() < maxInsertedBranchSize
-                            where branch.GetHeight() < maxInsertedBranchHeight
-                            // where crossoverPoint0.GetAllowedSymbols(replacedSubtreeIndex).Contains(branch.Symbol)
-                            where IsMatchingPointType(crossoverPoint0, replacedSubtreeIndex, branch)
-                            select branch;
+      var allowedBranches = (from branch in parent1.Root.IterateNodesPrefix()
+                             where branch.GetSize() < maxInsertedBranchSize
+                             where branch.GetHeight() < maxInsertedBranchHeight
+                             where IsMatchingPointType(crossoverPoint0, replacedSubtreeIndex, branch)
+                             select branch).ToList();
 
       if (allowedBranches.Count() == 0) {
@@ -90,34 +89,23 @@
 
     private static bool IsMatchingPointType(SymbolicExpressionTreeNode parent, int replacedSubtreeIndex, SymbolicExpressionTreeNode branch) {
-      // cannot compare dynamic symbols by reference because new instances of Invoke/Argument are created on demand
-      // instead we must check equality by names and number of arguments 
-      var branchSymbols = (from node in branch.IterateNodesPrefix()
-                           select new { FunctionName = node.Symbol.Name, SubtreeCount = node.SubTrees.Count }).Distinct();
+      // check point type for the whole branch
+      foreach (var node in branch.IterateNodesPostfix()) {
+        if (!parent.Grammar.ContainsSymbol(node.Symbol)) return false;
+        else if (node.SubTrees.Count < parent.Grammar.GetMinSubtreeCount(node.Symbol)) return false;
+        else if (node.SubTrees.Count > parent.Grammar.GetMaxSubtreeCount(node.Symbol)) return false;
+      }
 
       // check syntax constraints of direct parent - child relation
-      var matchingSymbolInParentGrammar = (from sym in parent.GetAllowedSymbols(replacedSubtreeIndex)
-                                           where sym.Name == branch.Symbol.Name
-                                           select sym).SingleOrDefault();
-      if (matchingSymbolInParentGrammar == null ||
-        branch.SubTrees.Count < parent.Grammar.GetMinSubtreeCount(matchingSymbolInParentGrammar) ||
-        branch.SubTrees.Count > parent.Grammar.GetMaxSubtreeCount(matchingSymbolInParentGrammar)) return false;
-
-      // check point type for the whole branch
-      foreach (var branchSymbol in branchSymbols) {
-        matchingSymbolInParentGrammar = (from sym in parent.Grammar.Symbols
-                                         where sym.Name == branchSymbol.FunctionName
-                                         select sym).SingleOrDefault();
-        if (matchingSymbolInParentGrammar == null ||
-          branchSymbol.SubtreeCount < parent.Grammar.GetMinSubtreeCount(matchingSymbolInParentGrammar) ||
-          branchSymbol.SubtreeCount > parent.Grammar.GetMaxSubtreeCount(matchingSymbolInParentGrammar)) return false;
-      }
+      if (!parent.Grammar.IsAllowedChild(parent.Symbol, branch.Symbol, replacedSubtreeIndex)) return false;
 
       return true;
     }
 
-    private static void SelectCrossoverPoint(IRandom random, SymbolicExpressionTree parent0, double internalNodeProbability, out SymbolicExpressionTreeNode crossoverPoint, out int subtreeIndex) {
+    private static void SelectCrossoverPoint(IRandom random, SymbolicExpressionTree parent0, double internalNodeProbability, int maxBranchSize, int maxBranchHeight, out SymbolicExpressionTreeNode crossoverPoint, out int subtreeIndex) {
       var crossoverPoints = from branch in parent0.Root.IterateNodesPrefix()
                             where branch.SubTrees.Count > 0
                             where branch != parent0.Root
+                            where branch.GetSize() < maxBranchSize
+                            where branch.GetHeight() < maxBranchHeight
                             from index in Enumerable.Range(0, branch.SubTrees.Count)
                             let p = new { CrossoverPoint = branch, SubtreeIndex = index, IsLeaf = branch.SubTrees[index].SubTrees.Count == 0 }
@@ -151,7 +139,7 @@
     private static SymbolicExpressionTreeNode SelectRandomBranch(IRandom random, IEnumerable<SymbolicExpressionTreeNode> branches, double internalNodeProbability) {
       if (internalNodeProbability < 0.0 || internalNodeProbability > 1.0) throw new ArgumentException("internalNodeProbability");
-      var groupedBranches = from branch in branches
-                            group branch by branch.SubTrees.Count into g
-                            select g;
+      var groupedBranches = (from branch in branches
+                             group branch by branch.SubTrees.Count into g
+                             select g).ToList();
       var allowedInternalBranches = (from g in groupedBranches
                                      where g.Key > 0
@@ -163,12 +151,12 @@
                                  select leaf).ToList();
       if (allowedInternalBranches.Count == 0) {
-        return allowedLeafBranches[random.Next(allowedLeafBranches.Count)];
+        return allowedLeafBranches.SelectRandom(random);
       } else if (allowedLeafBranches.Count == 0) {
-        return allowedInternalBranches[random.Next(allowedInternalBranches.Count)];
+        return allowedInternalBranches.SelectRandom(random);
       } else if (random.NextDouble() < internalNodeProbability) {
         // when leaf and internal nodes are possible then choose either a leaf or internal node with internalNodeProbability
-        return allowedInternalBranches[random.Next(allowedInternalBranches.Count)];
+        return allowedInternalBranches.SelectRandom(random);
       } else {
-        return allowedLeafBranches[random.Next(allowedLeafBranches.Count)];
+        return allowedLeafBranches.SelectRandom(random);
       }
     }
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/DefaultSymbolicExpressionGrammar.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/DefaultSymbolicExpressionGrammar.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/DefaultSymbolicExpressionGrammar.cs	(revision 3369)
@@ -30,4 +30,10 @@
 
 namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
+  /// <summary>
+  /// The default symbolic expression grammar stores symbols and syntactic constraints for symbols.
+  /// Symbols are treated as equvivalent if they have the same name.
+  /// Syntactic constraints limit the number of allowed sub trees for a node with a symbol and which symbols are allowed 
+  /// in the sub-trees of a symbol (can be specified for each sub-tree index separately).
+  /// </summary>
   [StorableClass]
   [Item("DefaultSymbolicExpressionGrammar", "Represents a grammar that defines the syntax of symbolic expression trees.")]
@@ -40,5 +46,5 @@
     private Dictionary<string, List<HashSet<string>>> allowedChildSymbols;
     [Storable]
-    private HashSet<Symbol> allSymbols;
+    private Dictionary<string, Symbol> allSymbols;
 
     public DefaultSymbolicExpressionGrammar()
@@ -66,5 +72,5 @@
       maxSubTreeCount = new Dictionary<string, int>();
       allowedChildSymbols = new Dictionary<string, List<HashSet<string>>>();
-      allSymbols = new HashSet<Symbol>();
+      allSymbols = new Dictionary<string, Symbol>();
       cachedMinExpressionLength = new Dictionary<string, int>();
       cachedMaxExpressionLength = new Dictionary<string, int>();
@@ -74,6 +80,6 @@
 
     public void AddSymbol(Symbol symbol) {
-      if (allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Symbol " + symbol + " is already defined.");
-      allSymbols.Add(symbol);
+      if (ContainsSymbol(symbol)) throw new ArgumentException("Symbol " + symbol + " is already defined.");
+      allSymbols.Add(symbol.Name, symbol);
       allowedChildSymbols[symbol.Name] = new List<HashSet<string>>();
       ClearCaches();
@@ -86,5 +92,5 @@
             allowedChildSymbols[parent.Name][i].Remove(symbol.Name);
       }
-      allSymbols.RemoveWhere(s => s.Name == symbol.Name);
+      allSymbols.Remove(symbol.Name);
       minSubTreeCount.Remove(symbol.Name);
       maxSubTreeCount.Remove(symbol.Name);
@@ -94,10 +100,14 @@
 
     public IEnumerable<Symbol> Symbols {
-      get { return allSymbols.AsEnumerable(); }
+      get { return allSymbols.Values.AsEnumerable(); }
+    }
+
+    public bool ContainsSymbol(Symbol symbol) {
+      return allSymbols.ContainsKey(symbol.Name);
     }
 
     public void SetAllowedChild(Symbol parent, Symbol child, int argumentIndex) {
-      if (!allSymbols.Any(s => s.Name == parent.Name)) throw new ArgumentException("Unknown symbol: " + parent, "parent");
-      if (!allSymbols.Any(s => s.Name == child.Name)) throw new ArgumentException("Unknown symbol: " + child, "child");
+      if (!ContainsSymbol(parent)) throw new ArgumentException("Unknown symbol: " + parent, "parent");
+      if (!ContainsSymbol(child)) throw new ArgumentException("Unknown symbol: " + child, "child");
       if (argumentIndex >= GetMaxSubtreeCount(parent)) throw new ArgumentException("Symbol " + parent + " can have only " + GetMaxSubtreeCount(parent) + " subtrees.");
       allowedChildSymbols[parent.Name][argumentIndex].Add(child.Name);
@@ -106,18 +116,17 @@
 
     public bool IsAllowedChild(Symbol parent, Symbol child, int argumentIndex) {
-      if (!allSymbols.Any(s => s.Name == parent.Name)) throw new ArgumentException("Unknown symbol: " + parent, "parent");
-      if (!allSymbols.Any(s => s.Name == child.Name)) throw new ArgumentException("Unknown symbol: " + child, "child");
+      if (!ContainsSymbol(parent)) throw new ArgumentException("Unknown symbol: " + parent, "parent");
+      if (!ContainsSymbol(child)) throw new ArgumentException("Unknown symbol: " + child, "child");
       if (argumentIndex >= GetMaxSubtreeCount(parent)) throw new ArgumentException("Symbol " + parent + " can have only " + GetMaxSubtreeCount(parent) + " subtrees.");
-      if (allowedChildSymbols.ContainsKey(parent.Name)) return allowedChildSymbols[parent.Name][argumentIndex].Contains(child.Name);
-      return false;
+      return allowedChildSymbols[parent.Name][argumentIndex].Contains(child.Name);
     }
 
     private Dictionary<string, int> cachedMinExpressionLength;
     public int GetMinExpressionLength(Symbol symbol) {
-      if (!allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Unknown symbol: " + symbol);
+      if (!ContainsSymbol(symbol)) throw new ArgumentException("Unknown symbol: " + symbol);
       if (!cachedMinExpressionLength.ContainsKey(symbol.Name)) {
         cachedMinExpressionLength[symbol.Name] = int.MaxValue; // prevent infinite recursion
         long sumOfMinExpressionLengths = 1 + (from argIndex in Enumerable.Range(0, GetMinSubtreeCount(symbol))
-                                              let minForSlot = (long)(from s in allSymbols
+                                              let minForSlot = (long)(from s in Symbols
                                                                       where IsAllowedChild(symbol, s, argIndex)
                                                                       select GetMinExpressionLength(s)).DefaultIfEmpty(0).Min()
@@ -131,9 +140,9 @@
     private Dictionary<string, int> cachedMaxExpressionLength;
     public int GetMaxExpressionLength(Symbol symbol) {
-      if (!allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Unknown symbol: " + symbol);
+      if (!ContainsSymbol(symbol)) throw new ArgumentException("Unknown symbol: " + symbol);
       if (!cachedMaxExpressionLength.ContainsKey(symbol.Name)) {
         cachedMaxExpressionLength[symbol.Name] = int.MaxValue; // prevent infinite recursion
         long sumOfMaxTrees = 1 + (from argIndex in Enumerable.Range(0, GetMaxSubtreeCount(symbol))
-                                  let maxForSlot = (long)(from s in allSymbols
+                                  let maxForSlot = (long)(from s in Symbols
                                                           where IsAllowedChild(symbol, s, argIndex)
                                                           select GetMaxExpressionLength(s)).DefaultIfEmpty(0).Max()
@@ -147,9 +156,9 @@
     private Dictionary<string, int> cachedMinExpressionDepth;
     public int GetMinExpressionDepth(Symbol symbol) {
-      if (!allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Unknown symbol: " + symbol);
+      if (!ContainsSymbol(symbol)) throw new ArgumentException("Unknown symbol: " + symbol);
       if (!cachedMinExpressionDepth.ContainsKey(symbol.Name)) {
         cachedMinExpressionDepth[symbol.Name] = int.MaxValue; // prevent infinite recursion
         cachedMinExpressionDepth[symbol.Name] = 1 + (from argIndex in Enumerable.Range(0, GetMinSubtreeCount(symbol))
-                                                     let minForSlot = (from s in allSymbols
+                                                     let minForSlot = (from s in Symbols
                                                                        where IsAllowedChild(symbol, s, argIndex)
                                                                        select GetMinExpressionDepth(s)).DefaultIfEmpty(0).Min()
@@ -160,5 +169,5 @@
 
     public void SetMaxSubtreeCount(Symbol symbol, int nSubTrees) {
-      if (!allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Unknown symbol: " + symbol);
+      if (!ContainsSymbol(symbol)) throw new ArgumentException("Unknown symbol: " + symbol);
       maxSubTreeCount[symbol.Name] = nSubTrees;
       while (allowedChildSymbols[symbol.Name].Count <= nSubTrees)
@@ -171,5 +180,5 @@
 
     public void SetMinSubtreeCount(Symbol symbol, int nSubTrees) {
-      if (!allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Unknown symbol: " + symbol);
+      if (!ContainsSymbol(symbol)) throw new ArgumentException("Unknown symbol: " + symbol);
       minSubTreeCount[symbol.Name] = nSubTrees;
       ClearCaches();
@@ -177,10 +186,10 @@
 
     public int GetMinSubtreeCount(Symbol symbol) {
-      if (!allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Unknown symbol: " + symbol);
+      if (!ContainsSymbol(symbol)) throw new ArgumentException("Unknown symbol: " + symbol);
       return minSubTreeCount[symbol.Name];
     }
 
     public int GetMaxSubtreeCount(Symbol symbol) {
-      if (!allSymbols.Any(s => s.Name == symbol.Name)) throw new ArgumentException("Unknown symbol: " + symbol);
+      if (!ContainsSymbol(symbol)) throw new ArgumentException("Unknown symbol: " + symbol);
       return maxSubTreeCount[symbol.Name];
     }
@@ -193,18 +202,4 @@
       cachedMinExpressionDepth.Clear();
     }
-
-    //private void symbol_ToStringChanged(object sender, EventArgs e) {
-    //  OnToStringChanged();
-    //}
-
-    //private bool IsValidExpression(SymbolicExpressionTreeNode root) {
-    //  if (root.SubTrees.Count < root.GetMinSubtreeCount()) return false;
-    //  else if (root.SubTrees.Count > root.GetMaxSubtreeCount()) return false;
-    //  else for (int i = 0; i < root.SubTrees.Count; i++) {
-    //      if (!root.GetAllowedSymbols(i).Select(x => x.Name).Contains(root.SubTrees[i].Symbol.Name)) return false;
-    //      if (!IsValidExpression(root.SubTrees[i])) return false;
-    //    }
-    //  return true;
-    //}
 
     public override IDeepCloneable Clone(Cloner cloner) {
@@ -220,5 +215,5 @@
         }
       }
-      clone.allSymbols = new HashSet<Symbol>(allSymbols);
+      clone.allSymbols = new Dictionary<string, Symbol>(allSymbols);
       return clone;
     }
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding-3.3.csproj
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding-3.3.csproj	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding-3.3.csproj	(revision 3369)
@@ -99,5 +99,4 @@
     <Compile Include="GlobalSymbolicExpressionGrammar.cs" />
     <Compile Include="EnumerableExtensions.cs" />
-    <Compile Include="Symbols\StartSymbolTreeNode.cs" />
     <Compile Include="DefaultSymbolicExpressionGrammar.cs">
       <SubType>Code</SubType>
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Interfaces/ISymbolicExpressionGrammar.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Interfaces/ISymbolicExpressionGrammar.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Interfaces/ISymbolicExpressionGrammar.cs	(revision 3369)
@@ -34,4 +34,5 @@
     void RemoveSymbol(Symbol symbol);
     IEnumerable<Symbol> Symbols { get; }
+    bool ContainsSymbol(Symbol symbol);
     void SetAllowedChild(Symbol parent, Symbol child, int argumentIndex);
     bool IsAllowedChild(Symbol parent, Symbol child, int argumentIndex);
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeCrossover.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeCrossover.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeCrossover.cs	(revision 3369)
@@ -81,7 +81,4 @@
       if (!success) FailedCrossoverEvents.Value++;
 
-      Debug.Assert(result.Size <= MaxTreeSizeParameter.ActualValue.Value);
-      Debug.Assert(result.Height <= MaxTreeHeightParameter.ActualValue.Value);
-      // Debug.Assert(grammar.IsValidExpression(result));
       ChildParameter.ActualValue = result;
       return base.Apply();
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeNode.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeNode.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeNode.cs	(revision 3369)
@@ -37,4 +37,6 @@
     [Storable]
     private Symbol symbol;
+    //[Storable]
+    private SymbolicExpressionTreeNode parent;
 
     public SymbolicExpressionTreeNode() {
@@ -51,7 +53,6 @@
       symbol = original.symbol;
       subTrees = new List<SymbolicExpressionTreeNode>();
-      grammar = original.grammar;
       foreach (var subtree in original.SubTrees) {
-        SubTrees.Add((SymbolicExpressionTreeNode)subtree.Clone());
+        AddSubTree((SymbolicExpressionTreeNode)subtree.Clone());
       }
     }
@@ -70,12 +71,12 @@
     }
 
-    private ISymbolicExpressionGrammar grammar;
-    public virtual ISymbolicExpressionGrammar Grammar {
-      get { return grammar; }
-      set {
-        grammar = value;
-        foreach (var subtree in subTrees)
-          subtree.Grammar = value;
-      }
+    internal SymbolicExpressionTreeNode Parent {
+      get { return parent; }
+      set { parent = value; }
+    }
+
+    internal virtual ISymbolicExpressionGrammar Grammar {
+      get { return parent.Grammar; }
+      set { throw new NotSupportedException("Grammar can be set only for SymbolicExpressionTreeTopLevelNodes."); }
     }
 
@@ -96,19 +97,16 @@
 
     public virtual void AddSubTree(SymbolicExpressionTreeNode tree) {
-      SubTrees.Add(tree);
-      //if (tree != null) 
-      tree.Grammar = Grammar;
+      subTrees.Add(tree);
+      tree.Parent = this;
     }
 
     public virtual void InsertSubTree(int index, SymbolicExpressionTreeNode tree) {
-      SubTrees.Insert(index, tree);
-      //if (tree != null) 
-      tree.Grammar = Grammar;
+      subTrees.Insert(index, tree);
+      tree.Parent = this;
     }
 
     public virtual void RemoveSubTree(int index) {
-      //if (SubTrees[index] != null)
-      SubTrees[index].Grammar = null;
-      SubTrees.RemoveAt(index);
+      subTrees[index].Parent = null;
+      subTrees.RemoveAt(index);
     }
 
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeTopLevelNode.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeTopLevelNode.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/SymbolicExpressionTreeTopLevelNode.cs	(revision 3369)
@@ -42,8 +42,26 @@
     }
 
+    private ISymbolicExpressionGrammar grammar;
+    //internal virtual ISymbolicExpressionGrammar Grammar {
+    //  get { return grammar; }
+    //  set {
+    //    grammar = value;
+    //    //foreach (var subtree in subTrees)
+    //    //  subtree.Grammar = value;
+    //  }
+    //}
+    internal override ISymbolicExpressionGrammar Grammar {
+      get {
+        return grammar;
+      }
+      set {
+        grammar = value;
+      }
+    }
+
     // copy constructor
     protected SymbolicExpressionTreeTopLevelNode(SymbolicExpressionTreeTopLevelNode original)
       : base(original) {
-      Grammar = (ISymbolicExpressionGrammar)original.Grammar.Clone();
+      grammar = (ISymbolicExpressionGrammar)original.Grammar.Clone();
     }
 
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/ArgumentTreeNode.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/ArgumentTreeNode.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/ArgumentTreeNode.cs	(revision 3369)
@@ -25,5 +25,5 @@
   [StorableClass]
   public sealed class ArgumentTreeNode : SymbolicExpressionTreeNode {
-    public new Argument Symbol {
+    internal new Argument Symbol {
       get { return (Argument)base.Symbol; }
       set {
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/DefunTreeNode.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/DefunTreeNode.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/DefunTreeNode.cs	(revision 3369)
@@ -39,8 +39,4 @@
     }
 
-    private new Defun Symbol {
-      get { return (Defun)base.Symbol; }
-    }
-
     // copy constructor
     private DefunTreeNode(DefunTreeNode original)
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/InvokeFunctionTreeNode.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/InvokeFunctionTreeNode.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/InvokeFunctionTreeNode.cs	(revision 3369)
@@ -27,5 +27,5 @@
     public new InvokeFunction Symbol {
       get { return (InvokeFunction)base.Symbol; }
-      set {
+      internal set {
         if (value == null) throw new ArgumentNullException();
         if (!(value is InvokeFunction)) throw new ArgumentNullException();
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/ProgramRootSymbol.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/ProgramRootSymbol.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/ProgramRootSymbol.cs	(revision 3369)
@@ -31,4 +31,8 @@
       }
     }
+
+    public override SymbolicExpressionTreeNode CreateTreeNode() {
+      return new SymbolicExpressionTreeTopLevelNode(this);
+    }
   }
 }
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/StartSymbol.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/StartSymbol.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Symbols/StartSymbol.cs	(revision 3369)
@@ -33,5 +33,5 @@
 
     public override SymbolicExpressionTreeNode CreateTreeNode() {
-      return new StartSymbolTreeNode(this);
+      return new SymbolicExpressionTreeTopLevelNode(this);
     }
   }
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/AllArchitectureAlteringOperatorsTest.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/AllArchitectureAlteringOperatorsTest.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/AllArchitectureAlteringOperatorsTest.cs	(revision 3369)
@@ -74,11 +74,21 @@
       for (int g = 0; g < N_ITERATIONS; g++) {
         for (int i = 0; i < POPULATION_SIZE; i++) {
-          var selectedTree = (SymbolicExpressionTree)trees.SelectRandom(random).Clone();
-          var op = combinedAAOperator.Operators.SelectRandom(random);
-          bool success;
-          op.ModifyArchitecture(random, selectedTree, grammar, maxTreeSize, maxTreeHeigth, maxDefuns, maxArgs, out success);
-          if (!success) failedEvents++;
-          Util.IsValid(selectedTree);
-          newTrees.Add(selectedTree);
+          if (random.NextDouble() < 0.5) {
+            // manipulate
+            var selectedTree = (SymbolicExpressionTree)trees.SelectRandom(random).Clone();
+            var op = combinedAAOperator.Operators.SelectRandom(random);
+            bool success;
+            op.ModifyArchitecture(random, selectedTree, grammar, maxTreeSize, maxTreeHeigth, maxDefuns, maxArgs, out success);
+            if (!success) failedEvents++;
+            Util.IsValid(selectedTree);
+            newTrees.Add(selectedTree);
+          } else {
+            // crossover
+            var par0 = (SymbolicExpressionTree)trees.SelectRandom(random).Clone();
+            var par1 = (SymbolicExpressionTree)trees.SelectRandom(random).Clone();
+            bool success;
+            newTrees.Add(SubtreeCrossover.Cross(random, par0, par1, 0.9, 100, 10, out success));
+            if (!success) failedEvents++;
+          }
         }
         trees = newTrees;
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/Grammars.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/Grammars.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/Grammars.cs	(revision 3369)
@@ -97,23 +97,23 @@
 
     public static void HasValidAdfGrammars(SymbolicExpressionTree tree) {
-      Assert.AreEqual(tree.Root.Grammar.Symbols.Count(), 8);
-      Assert.AreEqual(tree.Root.GetAllowedSymbols(0).Count(), 1); // only the start symbol is allowed
-      // we allow 3 ADF branches
-      Assert.AreEqual(tree.Root.GetAllowedSymbols(1).Count(), 1); // only the defun branch is allowed
-      Assert.AreEqual(tree.Root.GetAllowedSymbols(2).Count(), 1); // only the defun symbol is allowed
-      Assert.AreEqual(tree.Root.GetAllowedSymbols(3).Count(), 1); // only the defun symbol is allowed
-      foreach (var subtree in tree.Root.SubTrees) {
-        // check consistency of each sub-tree grammar independently
-        var allowedSymbols = subtree.GetAllowedSymbols(0);
-        int numberOfAllowedSymbols = allowedSymbols.Count();
-        foreach (var parent in allowedSymbols) {
-          for (int argIndex = 0; argIndex < subtree.Grammar.GetMaxSubtreeCount(parent); argIndex++) {
-            var allowedChildren = from child in subtree.Grammar.Symbols
-                                  where subtree.Grammar.IsAllowedChild(parent, child, argIndex)
-                                  select child;
-            Assert.AreEqual(numberOfAllowedSymbols, allowedChildren.Count());
-          }
-        }
-      }
+      //Assert.AreEqual(tree.Root.Grammar.Symbols.Count(), 8);
+      //Assert.AreEqual(tree.Root.GetAllowedSymbols(0).Count(), 1); // only the start symbol is allowed
+      //// we allow 3 ADF branches
+      //Assert.AreEqual(tree.Root.GetAllowedSymbols(1).Count(), 1); // only the defun branch is allowed
+      //Assert.AreEqual(tree.Root.GetAllowedSymbols(2).Count(), 1); // only the defun symbol is allowed
+      //Assert.AreEqual(tree.Root.GetAllowedSymbols(3).Count(), 1); // only the defun symbol is allowed
+      //foreach (var subtree in tree.Root.SubTrees) {
+      //  // check consistency of each sub-tree grammar independently
+      //  var allowedSymbols = subtree.GetAllowedSymbols(0);
+      //  int numberOfAllowedSymbols = allowedSymbols.Count();
+      //  foreach (var parent in allowedSymbols) {
+      //    for (int argIndex = 0; argIndex < subtree.Grammar.GetMaxSubtreeCount(parent); argIndex++) {
+      //      var allowedChildren = from child in subtree.Grammar.Symbols
+      //                            where subtree.Grammar.IsAllowedChild(parent, child, argIndex)
+      //                            select child;
+      //      Assert.AreEqual(numberOfAllowedSymbols, allowedChildren.Count());
+      //    }
+      //  }
+      //}
     }
 
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/ProbabilisticTreeCreaterTest.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/ProbabilisticTreeCreaterTest.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/ProbabilisticTreeCreaterTest.cs	(revision 3369)
@@ -54,5 +54,5 @@
       var randomTrees = new List<SymbolicExpressionTree>();
       var grammar = Grammars.CreateSimpleArithmeticGrammar();
-      var random = new MersenneTwister();
+      var random = new MersenneTwister(31415);
       for (int i = 0; i < POPULATION_SIZE; i++) {
         randomTrees.Add(ProbabilisticTreeCreator.Create(random, grammar, MAX_TREE_SIZE, MAX_TREE_HEIGHT, 0, 0));
@@ -75,5 +75,5 @@
       var randomTrees = new List<SymbolicExpressionTree>();
       var grammar = Grammars.CreateArithmeticAndAdfGrammar();
-      var random = new MersenneTwister();
+      var random = new MersenneTwister(31415);
       for (int i = 0; i < POPULATION_SIZE; i++) {
         var tree = ProbabilisticTreeCreator.Create(random, grammar, MAX_TREE_SIZE, MAX_TREE_HEIGHT, 3, 3);
Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/Util.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/Util.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Tests/Util.cs	(revision 3369)
@@ -117,16 +117,16 @@
     public static void IsValid(SymbolicExpressionTree tree) {
       Grammars.HasValidAdfGrammars(tree);
-      Assert.AreEqual(tree.Root.Symbol, tree.Root.Grammar.StartSymbol);
-      foreach (var subtree in tree.Root.SubTrees)
-        Assert.AreNotSame(subtree.Grammar, tree.Root.Grammar);
+      //Assert.AreEqual(tree.Root.Symbol, tree.Root.Grammar.StartSymbol);
+      //foreach (var subtree in tree.Root.SubTrees)
+      //  Assert.AreNotSame(subtree.Grammar, tree.Root.Grammar);
       IsValid(tree.Root);
     }
 
     public static void IsValid(SymbolicExpressionTreeNode treeNode) {
-      var matchingSymbol = (from symb in treeNode.Grammar.Symbols
-                            where symb.Name == treeNode.Symbol.Name
-                            select symb).SingleOrDefault();
-      Assert.IsTrue(treeNode.SubTrees.Count >= treeNode.Grammar.GetMinSubtreeCount(matchingSymbol));
-      Assert.IsTrue(treeNode.SubTrees.Count <= treeNode.Grammar.GetMaxSubtreeCount(matchingSymbol));
+      //var matchingSymbol = (from symb in treeNode.Grammar.Symbols
+      //                      where symb.Name == treeNode.Symbol.Name
+      //                      select symb).SingleOrDefault();
+      //Assert.IsTrue(treeNode.SubTrees.Count >= treeNode.Grammar.GetMinSubtreeCount(matchingSymbol));
+      //Assert.IsTrue(treeNode.SubTrees.Count <= treeNode.Grammar.GetMaxSubtreeCount(matchingSymbol));
       for (int i = 0; i < treeNode.SubTrees.Count; i++) {
         Assert.IsTrue(treeNode.GetAllowedSymbols(i).Select(x => x.Name).Contains(treeNode.SubTrees[i].Symbol.Name));

Index: /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Creators/ProbabilisticTreeCreator.cs
===================================================================
--- /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Creators/ProbabilisticTreeCreator.cs	(revision 3368)
+++ /trunk/sources/HeuristicLab.Encodings.SymbolicExpressionTreeEncoding/3.3/Creators/ProbabilisticTreeCreator.cs	(revision 3369)
@@ -35,4 +35,5 @@
   [Item("ProbabilisticTreeCreator", "An operator that creates new symbolic expression trees with uniformly distributed size")]
   public class ProbabilisticTreeCreator : SymbolicExpressionTreeCreator {
+    private const int MAX_TRIES = 100;
 
     public ProbabilisticTreeCreator()
@@ -53,5 +54,7 @@
       ) {
       SymbolicExpressionTree tree = new SymbolicExpressionTree();
-      tree.Root = PTC2(random, grammar, grammar.StartSymbol, maxTreeSize, maxTreeHeight, maxFunctionDefinitions, maxFunctionArguments);
+      var rootNode = grammar.StartSymbol.CreateTreeNode();
+      rootNode.Grammar = grammar;
+      tree.Root = PTC2(random, rootNode, maxTreeSize, maxTreeHeight, maxFunctionDefinitions, maxFunctionArguments);
       return tree;
     }
@@ -63,43 +66,43 @@
     }
 
-    /// <summary>
-    /// Creates a random tree with <paramref name="maxTreeSize"/> and <paramref name="maxDepth"/>.
-    /// </summary>
-    /// <param name="random"></param>
-    /// <param name="grammar"></param>
-    /// <param name="rootSymbol"></param>
-    /// <param name="maxTreeSize"></param>
-    /// <param name="maxDepth"></param>
-    /// <param name="maxFunctionDefinitions"></param>
-    /// <param name="maxFunctionArguments"></param>
-    /// <returns></returns>
-    public static SymbolicExpressionTreeNode PTC2(IRandom random, ISymbolicExpressionGrammar grammar, Symbol rootSymbol,
+    public static SymbolicExpressionTreeNode PTC2(IRandom random, SymbolicExpressionTreeNode seedNode,
       int maxTreeSize, int maxDepth, int maxFunctionDefinitions, int maxFunctionArguments) {
       // tree size is limited by the grammar and by the explicit size constraints
-      int allowedMinSize = grammar.GetMinExpressionLength(rootSymbol);
-      int allowedMaxSize = Math.Min(maxTreeSize, grammar.GetMaxExpressionLength(rootSymbol));
-      // select a target tree size uniformly in the possible range (as determined by explicit limits and limits of the grammar)
-      int treeSize = random.Next(allowedMinSize, allowedMaxSize + 1);
-      SymbolicExpressionTreeNode root = null;
-      do {
-        try {
-          root = rootSymbol.CreateTreeNode();
-          root.Grammar = grammar;
-          if (treeSize <= 1 || maxDepth <= 1) return root;
-          CreateFullTreeFromSeed(random, root, treeSize, maxDepth, maxFunctionDefinitions, maxFunctionArguments);
-        }
-        catch (ArgumentException) {
-          // try a different size
-          root = null;
-          treeSize = random.Next(allowedMinSize, allowedMaxSize);
-        }
-      } while (root == null || root.GetSize() > maxTreeSize || root.GetHeight() > maxDepth);
-      return root;
-    }
-
-    private static void CreateFullTreeFromSeed(IRandom random, SymbolicExpressionTreeNode root, int size, int maxDepth, int maxFunctionDefinitions, int maxFunctionArguments) {
+      int allowedMinSize = seedNode.Grammar.GetMinExpressionLength(seedNode.Symbol);
+      int allowedMaxSize = Math.Min(maxTreeSize, seedNode.Grammar.GetMaxExpressionLength(seedNode.Symbol));
+      int tries = 0;
+      while (tries++ < MAX_TRIES) {
+        // select a target tree size uniformly in the possible range (as determined by explicit limits and limits of the grammar)
+        int treeSize = random.Next(allowedMinSize, allowedMaxSize + 1);
+        if (treeSize <= 1 || maxDepth <= 1) return seedNode;
+
+        bool success = CreateFullTreeFromSeed(random, seedNode, seedNode.Grammar, treeSize, maxDepth, maxFunctionDefinitions, maxFunctionArguments);
+
+        // if successfull => check constraints and return the tree if everything looks ok        
+        if (success && seedNode.GetSize() <= maxTreeSize && seedNode.GetHeight() <= maxDepth) {
+          return seedNode;
+        } else {
+          // clean seedNode
+          while (seedNode.SubTrees.Count > 0) seedNode.RemoveSubTree(0);
+        }
+        // try a different size MAX_TRIES times
+      }
+      throw new ArgumentException("Couldn't create a valid tree with the specified constraints.");
+    }
+
+    private static bool CreateFullTreeFromSeed(IRandom random, SymbolicExpressionTreeNode root, ISymbolicExpressionGrammar globalGrammar,
+      int size, int maxDepth, int maxFunctionDefinitions, int maxFunctionArguments) {
+      try {
+        TryCreateFullTreeFromSeed(random, root, globalGrammar, size, maxDepth, maxFunctionDefinitions, maxFunctionArguments);
+        return true;
+      }
+      catch (ArgumentException) { return false; }
+    }
+
+    private static void TryCreateFullTreeFromSeed(IRandom random, SymbolicExpressionTreeNode root, ISymbolicExpressionGrammar globalGrammar,
+      int size, int maxDepth, int maxFunctionDefinitions, int maxFunctionArguments) {
       List<TreeExtensionPoint> extensionPoints = new List<TreeExtensionPoint>();
       int currentSize = 1;
-      int totalListMinSize = root.Grammar.GetMinExpressionLength(root.Symbol) - 1;
+      int totalListMinSize = globalGrammar.GetMinExpressionLength(root.Symbol) - 1;
       int actualArity = SampleArity(random, root, size);
       for (int i = 0; i < actualArity; i++) {
@@ -107,5 +110,5 @@
         var dummy = new SymbolicExpressionTreeNode();
         root.AddSubTree(dummy);
-        dummy.Grammar = (ISymbolicExpressionGrammar)dummy.Grammar.Clone();
+        // dummy.Grammar = (ISymbolicExpressionGrammar)dummy.Grammar.Clone();
         extensionPoints.Add(new TreeExtensionPoint { Parent = root, ChildIndex = i, ExtensionPointDepth = 2 });
       }
@@ -141,6 +144,6 @@
             var dummy = new SymbolicExpressionTreeNode();
             newTree.AddSubTree(dummy);
-            if (IsTopLevelBranch(root, dummy))
-              dummy.Grammar = (ISymbolicExpressionGrammar)dummy.Grammar.Clone();
+            //if (IsTopLevelBranch(root, dummy))
+            //  dummy.Grammar = (ISymbolicExpressionGrammar)dummy.Grammar.Clone();
             extensionPoints.Add(new TreeExtensionPoint { Parent = newTree, ChildIndex = i, ExtensionPointDepth = extensionDepth + 1 });
           }
@@ -175,5 +178,5 @@
         var dummy = new SymbolicExpressionTreeNode();
         tree.AddSubTree(dummy);
-        dummy.Grammar = (ISymbolicExpressionGrammar)dummy.Grammar.Clone();
+        // dummy.Grammar = (ISymbolicExpressionGrammar)dummy.Grammar.Clone();
         // replace the just inserted dummy by recursive application
         ReplaceWithMinimalTree(random, root, tree, i, maxFunctionDefinitions, maxFunctionArguments);
@@ -185,5 +188,5 @@
       // also assumes that newTree is already attached to root somewhere
       if (IsTopLevelBranch(root, newTree)) {
-        newTree.Grammar = (ISymbolicExpressionGrammar)newTree.Grammar.Clone();
+        newTree.Grammar = (ISymbolicExpressionGrammar)root.Grammar.Clone();
 
         // allow invokes of existing ADFs with higher index
@@ -225,5 +228,6 @@
 
     private static bool IsTopLevelBranch(SymbolicExpressionTreeNode root, SymbolicExpressionTreeNode branch) {
-      return root.SubTrees.IndexOf(branch) > -1;
+      //return root.SubTrees.IndexOf(branch) > -1;
+      return branch is SymbolicExpressionTreeTopLevelNode;
     }
 